@@ -97,6 +97,22 @@ data Config m a = Config
     , proxyType      :: Maybe ProxyType
     , startupHook    :: Maybe (StartupInfo m a -> IO ())
     }
+#if MIN_VERSION_base(4,7,0)
+  deriving Typeable
+#else
+
+
+------------------------------------------------------------------------------
+-- | The 'Typeable1' instance is here so 'Config' values can be
+-- dynamically loaded with Hint.
+configTyCon :: TyCon
+configTyCon = mkTyCon "Snap.Http.Server.Config.Config"
+{-# NOINLINE configTyCon #-}
+
+instance (Typeable1 m) => Typeable1 (Config m) where
+    typeOf1 _ = mkTyConApp configTyCon [typeOf1 (undefined :: m ())]
+#endif
+
 
 instance Show (Config m a) where
     show c = unlines [ "Config:"
@@ -189,17 +205,6 @@ instance Monoid (Config m a) where
         ov f = getLast $! (mappend `on` (Last . f)) a b
 
 
-------------------------------------------------------------------------------
--- | The 'Typeable1' instance is here so 'Config' values can be
--- dynamically loaded with Hint.
-configTyCon :: TyCon
-configTyCon = mkTyCon "Snap.Http.Server.Config.Config"
-{-# NOINLINE configTyCon #-}
-
-instance (Typeable1 m) => Typeable1 (Config m) where
-    typeOf1 _ = mkTyConApp configTyCon [typeOf1 (undefined :: m ())]
-
-
 ------------------------------------------------------------------------------
 -- | These are the default values for the options
 defaultConfig :: MonadSnap m => Config m a
